#include<vector>
#include<iostream>
#include "Eigen/Dense"
#include "CImg.h"
#include "snake.h"

using namespace Eigen;
using namespace std;
using namespace cimg_library;

int main(int argc,char **argv) {
    cimg_usage("Retrieve command line arguments");
	const char* filename = cimg_option("-i","../../images/1.bmp","Input image file");
    const double sigma   = cimg_option("-s",2,"Standard variation of the gaussian smoothing");
    const double gamma   = cimg_option("-g",0.002,"Gamma: magnitude of the time step");
    const double alpha   = cimg_option("-a",0.3,"Alpha: weight of the smoothing term");
    const double beta   = cimg_option("-b",0.1,"Beta: weight of the curvature term");
    
    CImg<unsigned char> img(filename);
    //// Blur the image (use the cimg library function)
	//.....
    snake s;
    //dispImage is a color image so that we can draw the contours in color
    CImg<unsigned char> dispImg(img.width(),img.height(),1,3,0);
    CImg<unsigned char> colorImg(img.width(),img.height(),1,3,0);

    CImg<unsigned char> grayImg;
    //if it's a color image, convert it to grayscale
    if(img.spectrum()==3)
    {
        CImgList<unsigned char> colors = (img<'c');
        grayImg = 0.3*colors(0)+0.59*colors(1)+0.11*colors(2);
        dispImg = img;
    }
    else
    {
        grayImg = img;
        dispImg += img; //dispImage is a color image, each channel is just a copy of the grayscale image
    }
    colorImg = dispImg;

    vector< vector<int> > points;
    const unsigned char red[] = { 255,0,0 }, green[] = { 0,255,0 }, blue[] = { 0,0,255 };

    dispImg.draw_text(5,5,"Draw points for contour - right click for last point",blue,0,1,13);
    CImgDisplay disp(dispImg,"Snakes active contours");

    const unsigned int nb_iterations = 5000;
    bool drawing = true;

    //draw the points
    while (!disp.is_closed()&&!disp.is_keyESC()&&(drawing==true)) {
        disp.wait();
        if (disp.button()&1 && disp.mouse_x()>=0 && disp.mouse_y()>=0) {
            vector<int> v(2,0);
            v[0]=disp.mouse_x(); v[1]=disp.mouse_y();
            dispImg.draw_circle(v[0], v[1], 2, red).display(disp);
            points.push_back(v);
        }

        if(disp.button()&2)
        {
            vector<int> v(2,0);
            v[0]=disp.mouse_x(); v[1]=disp.mouse_y();
            dispImg.draw_circle(v[0], v[1], 2, red).display(disp);
            points.push_back(v);

            for(int i = 0; i<points.size()-1; i++)
                dispImg.draw_line(points[i][0],points[i][1],points[i+1][0],points[i+1][1],green).display(disp);
            dispImg.draw_line(points[points.size()-1][0],points[points.size()-1][1],points[0][0],points[0][1],green).display(disp);
            drawing=false;
        }

        if (disp.is_resized()) disp.resize(disp);
    }

    s.assign(points, grayImg);
    s.setParams(gamma,beta,alpha);

    //show the evolution of the snake
    while(!disp.is_closed() && !disp.is_keyESC())
    {
        dispImg.draw_text(5,dispImg.height()-20,"For 5000 iterations, press I, to finish, press ESC",blue,0,1,13).display(disp);
        if(disp.is_keyI())
        {
            for (unsigned int iteration = 0; iteration<nb_iterations && !disp.is_closed() &&
                 !disp.is_keyS() && !disp.is_keyESC(); ++iteration)
            {
                s.update();
                dispImg = colorImg;
                s.display(dispImg,disp,red, green);
                dispImg.draw_text(5,5,"Iteration %d",blue,0,1,13,iteration).draw_text(5,dispImg.height()-20,"To stop iterations press S, press ESC to close windows",blue,0,1,13).display(disp);
            }
            dispImg = colorImg;
            s.display(dispImg,disp,red, green);
        }
    }
    return 0;
}
